<?xml version='1.0'?>

<!--
	XSL stylesheet to do post-processing of XML MDDataset result
-->
<xsl:stylesheet 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
	xmlns:msxsl="urn:schemas-microsoft-com:xslt" 
	xmlns:xa="urn:schemas-microsoft-com:xml-analysis:mddataset" 
	version="1.0">

	<xsl:output method="html"/>

	<!-- Create indexes -->
	<xsl:key name="AxisInfo" match="xa:root/xa:OlapInfo/xa:AxesInfo/xa:AxisInfo" use="@name"/>
	<xsl:key name="Axis" match="xa:root/xa:Axes/xa:Axis" use="@name"/>
	<xsl:key name="Cell" match="xa:root/xa:CellData/xa:Cell" use="@CellOrdinal"/>

	<!-- Remove loop invariant -->
	<xsl:variable name="varAxisTuples" select="key('Axis', 'Axis1')/xa:Tuples/xa:Tuple" />

	<!-- Root -->
	<xsl:template match="xa:root">
		<HTML>
		<HEAD>
		<STYLE TYPE="text/css">
		TH {font-family:Arial,sans-serif; font-size:12px; font-weight:bold; text-align:left; background-color: #333366; color: #FFFFFF;}
		</STYLE>
		</HEAD>
		<BODY BGCOLOR="#6699CC" style="background-color: #6699CC; color: black; font-size 8pt; font-face: Arial; border: '1 solid'; border-top-color: '#CFCFCF'; border-left-color: '#CFCFCF'; border-right-color: '#505050'; border-bottom-color: '#505050';">
		<xsl:choose>
			<xsl:when test="key('AxisInfo', 'Axis2')">
				<H3>Only two axes are supported!</H3>
			</xsl:when>
			<xsl:otherwise>
				<TABLE border="0" cellspacing="1">

				<!-- Populate the table -->
				<xsl:if test="key('AxisInfo', 'Axis0')">
					<xsl:apply-templates select="key('Axis', 'Axis0')" />
					<xsl:if test="key('AxisInfo', 'Axis1')">
						<xsl:apply-templates select="key('Axis', 'Axis1')" />
					</xsl:if>
				</xsl:if>

				<!-- End the table -->
				</TABLE>
			</xsl:otherwise>
		</xsl:choose>
		</BODY>
		</HTML>
	</xsl:template>

	<!-- Columns axis -->
	<xsl:template match="xa:Axis[@name='Axis0']">
		<!-- Show columns -->
		<xsl:variable name="varAxisTuples" select="xa:Tuples/xa:Tuple" />
		<xsl:for-each select="key('AxisInfo', 'Axis0')/xa:HierarchyInfo">
			<xsl:variable name="varHierarchy" select="." />
			<TR>
			<!-- Put blank <TH> tags for row axis dimensions -->
			<xsl:if test="key('AxisInfo', 'Axis1') and position() = 1">
				<TH>
				<xsl:attribute name="COLSPAN">
					<xsl:value-of select="count(key('AxisInfo', 'Axis1')/xa:HierarchyInfo)" />
				</xsl:attribute>
				<xsl:attribute name="ROWSPAN">
					<xsl:value-of select="count(key('AxisInfo', 'Axis0')/xa:HierarchyInfo)" />
				</xsl:attribute>
				</TH>
			</xsl:if>
			<!-- Get the axis members for the column axis -->
			<xsl:for-each select="key('Axis', 'Axis0')/xa:Tuples/*">
				<xsl:variable name="varAxisMember" select="." />
				<xsl:variable name="varAxisMemberPrev" select="preceding-sibling::xa:Tuple[position()=1]" />

				<!-- If this member is the first instance of a caption for the dimension, print the caption -->
				<xsl:variable name="varMemDimCols" select="$varAxisMember/xa:Member[@Hierarchy = $varHierarchy/@name]" />
				<xsl:variable name="varMemCapValue" select="$varMemDimCols/xa:Caption" />
				<xsl:variable name="varPrevMemDimCols" select="$varAxisMemberPrev/xa:Member[@Hierarchy = $varHierarchy/@name]" />
				<xsl:variable name="varPrevMemCapValue" select="$varPrevMemDimCols/xa:Caption" />

				<xsl:if test="not($varMemCapValue = $varPrevMemCapValue)">
				<TH>
					<xsl:attribute name="COLSPAN">
						<xsl:call-template name="GetSpans">
							<xsl:with-param name="varListOfMembers" select="$varAxisTuples" />
							<xsl:with-param name="varIndexCurrMember" select="position()" />
							<xsl:with-param name="varHierarchy" select="$varHierarchy/@name" />
							<xsl:with-param name="varColumnExpectedValue" select="$varMemCapValue" />
						</xsl:call-template>
					</xsl:attribute>
					<xsl:value-of select="$varMemCapValue" />
				</TH>
				</xsl:if>
			</xsl:for-each>
			</TR>
		</xsl:for-each>
	</xsl:template>

	<!-- Show a row axis caption if necessary -->
	<xsl:template match="//xa:Axis[@name='Axis1']/xa:Tuples/xa:Tuple">
		<xsl:variable name="varAxisMember" select="." />
		<xsl:variable name="varAxisMemberPrev" select="preceding-sibling::xa:Tuple[position()=1]" />
		<xsl:variable name="varTupleOrdinal" select="position() - 1" />

		<TR>
		<!-- Check each row axis dimension -->
		<xsl:for-each select="key('AxisInfo', 'Axis1')/xa:HierarchyInfo">
			<xsl:variable name="varHierarchy" select="." />
			<!-- If this member is the first instance of a caption for the dimension, print the caption -->
			<xsl:variable name="varMemDimCols" select="$varAxisMember/xa:Member[@Hierarchy = $varHierarchy/@name]" />
			<xsl:variable name="varMemCapValue" select="$varMemDimCols/xa:Caption" />
			<xsl:variable name="varPrevMemDimCols" select="$varAxisMemberPrev/xa:Member[@Hierarchy = $varHierarchy/@name]" />
			<xsl:variable name="varPrevMemCapValue" select="$varPrevMemDimCols/xa:Caption" />

			<xsl:if test="not($varMemCapValue = $varPrevMemCapValue)">
				<TH STYLE="background-color:#333366; color:#FFFFFF;">
				<xsl:attribute name="ROWSPAN">
				<xsl:call-template name="GetSpans">
					<xsl:with-param name="varListOfMembers" select="$varAxisTuples" />
					<xsl:with-param name="varIndexCurrMember" select="$varTupleOrdinal+1" />
					<xsl:with-param name="varHierarchy" select="$varHierarchy/@name" />
					<xsl:with-param name="varColumnExpectedValue" select="$varMemCapValue" />
				</xsl:call-template>
				</xsl:attribute>
				<FONT FACE="fixed">
				<xsl:call-template name="DoIndent">
					<xsl:with-param name="varColLevelNumber" select="../xa:LNum" />
					<xsl:with-param name="varDimMember" select="$varMemDimCols" />
				</xsl:call-template>
				</FONT>
				<xsl:value-of select="$varMemCapValue" />
				</TH>
			</xsl:if>
		</xsl:for-each>

		<!-- Display the cells for this tuple -->
		<xsl:variable name="varAxis0Coords" select="count(key('Axis', 'Axis0')/xa:Tuples/xa:Tuple)" />
		<xsl:variable name="varStartCell" select="$varTupleOrdinal * $varAxis0Coords" />
		<xsl:variable name="varEndCell" select="($varTupleOrdinal+1) * $varAxis0Coords" />

		<xsl:call-template name="PrintCells">
			<xsl:with-param name="varStartCell" select="$varTupleOrdinal * $varAxis0Coords" />
			<xsl:with-param name="varEndCell" select="($varTupleOrdinal+1) * $varAxis0Coords" />
		</xsl:call-template>
		</TR>
	</xsl:template>

	<!-- Print out cells within the range of ordinals -->
	<xsl:template name="PrintCells">
		<xsl:param name="varStartCell">0</xsl:param>
		<xsl:param name="varEndCell">0</xsl:param>

		<xsl:if test="$varStartCell &lt; $varEndCell">
			<TD STYLE="background-color:#eeeecc; color:#000000;">
			<xsl:choose>
				<xsl:when test="key('Cell', $varStartCell)">
					<xsl:value-of select="key('Cell', $varStartCell)/xa:FmtValue" />
				</xsl:when>
				<xsl:otherwise>
					<I>null</I>
				</xsl:otherwise>
			</xsl:choose>
			</TD>
			<xsl:call-template name="PrintCells">
				<xsl:with-param name="varStartCell" select="$varStartCell + 1" />
				<xsl:with-param name="varEndCell" select="$varEndCell" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<!-- Indent the member captions using the level number -->
	<xsl:template name="Indent">
		<xsl:param name="varLevelNumber">0</xsl:param>
		<xsl:if test="$varLevelNumber &gt; 0">
			<xsl:text disable-output-escaping = "yes">&amp;nbsp;&amp;nbsp;&amp;nbsp;</xsl:text>
			<xsl:call-template name="Indent">
				<xsl:with-param name="varLevelNumber" select="$varLevelNumber - 1" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<!-- Recursive template for doing member indentation -->
	<xsl:template name="DoIndent">
		<xsl:param name="varColLevelNumber">0</xsl:param>
		<xsl:param name="varDimMember">0</xsl:param>
		<xsl:if test="$varDimMember/xa:LNum &gt; 0">
			<xsl:call-template name="Indent">
				<xsl:with-param name="varLevelNumber" select="$varDimMember/xa:LNum" />
			</xsl:call-template>
		</xsl:if>
	</xsl:template>

	<!-- Recursive template to calculate the ROWSPANs/COLSPANs of a member caption -->
	<xsl:template name="GetSpans">
		<xsl:param name="varInitialSpans">0</xsl:param>
		<xsl:param name="varListOfMembers">0</xsl:param>
		<xsl:param name="varIndexCurrMember">0</xsl:param>
		<xsl:param name="varHierarchy">null</xsl:param>
		<xsl:param name="varColumnExpectedValue">0</xsl:param>

		<xsl:variable name="varDimMember" select="$varListOfMembers[$varIndexCurrMember]/xa:Member[@Hierarchy = $varHierarchy]" />
		<xsl:variable name="varColumnValue" select="$varDimMember/xa:Caption" />
		<xsl:choose>
			<xsl:when test="not($varColumnValue = $varColumnExpectedValue)">
				<xsl:value-of select="$varInitialSpans" />
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="GetSpans">
					<xsl:with-param name="varInitialSpans" select="1+$varInitialSpans" />
					<xsl:with-param name="varListOfMembers" select="$varListOfMembers" />
					<xsl:with-param name="varIndexCurrMember" select="1+$varIndexCurrMember" />
					<xsl:with-param name="varHierarchy" select="$varHierarchy" />
					<xsl:with-param name="varColumnExpectedValue" select="$varColumnExpectedValue" />
				</xsl:call-template>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

</xsl:stylesheet>

